#!/usr/bin/python
import sys
sys.path.append('lib/')
import os
import pickle

# ############### Input #################
# 1) File containing original QV's
# 2) File with dictionary
# 3) File with p
# 4) File with distortion
# 5) File containing reconstructed QV's from the lossy compressor
# 6) Output file
#########################################

# ############## Output #################
# 1) File with the new QV's
#########################################


if (len(sys.argv) == 1):
  print >> sys.stderr, "Program use:"
  print >> sys.stderr, sys.argv[0] + " <file with original QV's> <file w dictionary> <file with p> <file with distortions> <file with reconstructed QV's> <outputFile>" 
  sys.exit()

# Get list of files
try:
  inFile_ori = open(sys.argv[1])
except IOError:
  print >> sys.stderr, "Input file " + sys.argv[1] + " not found!"
  sys.exit()

try:
  inFile_rec = open(sys.argv[5])
except IOError:
  print >> sys.stderr, "Input file " + sys.argv[5] + " not found!"
  sys.exit()

print "loading dictionary"
tmp = pickle.load( open(sys.argv[2], "rb") )
print "loading p"
p = pickle.load( open(sys.argv[3], "rb") )
print "loading distortion"
distortion = pickle.load( open(sys.argv[4], "rb") )


# #### Compute empirical distribution ####
minQV = 33
maxQV = 104
rangeQV = maxQV - minQV + 1
# rangeQV = 200

print "len tmp: %d" % len(tmp)

print "Computing empirical distribution"
numLines = 0

while 1:
  q_ori = inFile_ori.readline()
  q_rec = inFile_rec.readline()
  if not q_ori:
    break

  numLines += 1
  if (numLines%1000000 == 0):
    print numLines

  readlen = len(q_ori)-1
  # print "read len is: %d" % readlen

  for position in range(0,readlen):
    if position == 0:
      # First position
      tmpkey = (0, ord(q_ori[position]), ord(q_ori[position + 1]))
    elif position == readlen - 1:
      # Last position
      tmpkey = (ord(q_ori[position - 1]), ord(q_ori[position]), 0)
    else:
      # Other positions
      tmpkey = (ord(q_ori[position-1]), ord(q_ori[position]), ord(q_ori[position + 1]))

    # print "%d \n" % (ord(q_rec[position])-33)
    p[tmp[tmpkey]][ord(q_rec[position])-33] += 1

inFile_ori.close()
inFile_rec.close()

print "NumLines is: %d" % numLines

# Open original file again
try:
  inFile_ori = open(sys.argv[1])
except IOError:
  print >> sys.stderr, "Input file " + sys.argv[1] + " not found!"
  sys.exit()

# Open output file
outFile = open(sys.argv[6], 'w')

# Compute the denoising
print "Computing denoising"
denoising = {}
numLines = 0
while 1:
  q_ori = inFile_ori.readline()
  if not q_ori:
    break
  readlen = len(q_ori) - 1


  numLines += 1
  if (numLines%1000000 == 0):
    print numLines

  for position in range(0,readlen):
    if position == 0:
      # First position
      tmpkey = (0, ord(q_ori[0]), ord(q_ori[1]))
    elif position == readlen - 1:
      # last position
      tmpkey = (ord(q_ori[readlen - 2]), ord(q_ori[readlen - 1]), 0)
    else:
      # other positions
      tmpkey = (ord(q_ori[position-1]), ord(q_ori[position]), ord(q_ori[position+1]))

    if tmpkey in denoising:
      optQV = denoising[tmpkey]
    else:
      for xhat in range(minQV-33, maxQV+1-33):
        tmpValue = 0
        for x in range(minQV-33, maxQV+1-33):
          tmpValue = tmpValue + p[tmp[tmpkey]][x]*distortion[xhat][x]

        if xhat == minQV-33:
          optQV = xhat
          optValue = tmpValue
        else:
          if tmpValue < optValue:
            optQV = xhat
            optValue = tmpValue
        
      optQV = optQV + 33
      denoising[tmpkey] = optQV

    outFile.write("%s" % str(unichr(optQV)))

  outFile.write("\n")

inFile_ori.close()
outFile.close()

